@@ -1,7 +1,9 @@
 from __future__ import division
 
-from sympy import *
-from applpy import *
+from pylab import (plot, xlabel, ylabel, title, grid, arange,
+                   ion, ioff)
+from rv import *
+x,y,z,t,v=symbols('x y z t v')
 
 """
     A Probability Progamming Language (APPL) -- Python Edition
@@ -29,12 +31,6 @@ Plotting Module
 Defines procedures for plotting random variables
 
 """
-try:
-    from pylab import *
-except:
-    print 'WARNING: Plotting not currently enabled'
-    print 'Download matplotlib to enable plotting.'
-    print ''
     
 def mat_plot(funclist,suplist,lab1=None,lab2=None,ftype='continuous'):
     """
